//**********************
// DAC control with SPI
//
// for PCM1795, Texas Instruments
// (C)Copyright 2012 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#include "config.h"
#include "type.h"
#include "gpio.h"
#include "DAC.h"

void InitDAC()
{

	// PORT1_3
	IOCON_SWDIO_PIO1_3 = IOCON_SWDIO_PIO1_3_FUNC_GPIO | IOCON_SWDIO_PIO1_3_MODE_PULLUP | IOCON_SWDIO_PIO1_3_ADMODE_DIGITAL;

	/* Initialize port pins */
	DAC_nMS_OUT;		// set output
	DAC_MDI_OUT;		// set output
	DAC_MC_OUT;			// set output
	DAC_MDO_IN;			// set input
	DAC_nRST_OUT;		// set output

	DAC_nRST_H;			// nRES active=low pulse
	DAC_nMS_H;			// nMS active=low
	DAC_MDI_L;			// MDI data output
	DAC_MC_L;			// MDI clock

	DAC_nRST_L;			// 20ns
	DAC_nRST_H;

}


void WriteDAC(unsigned char a, unsigned char d)
{
	unsigned char i;

	a&=0x7f;	// to write

	DAC_nMS_L;			// nMS active=low

	for(i=0;i<8;i++)
	{
		if(a&0x80)
		{
			DAC_MDI_H;
		}
		else
		{
			DAC_MDI_L;
		}
		DAC_MC_H;
		a<<=1;
		DAC_MC_L;
	}

	for(i=0;i<8;i++)
	{
		if(d&0x80)
		{
			DAC_MDI_H;
		}
		else
		{
			DAC_MDI_L;
		}
		DAC_MC_H;
		d<<=1;
		DAC_MC_L;
	}

	DAC_nMS_H;			// nMS active=low

}


unsigned char ReadDAC(unsigned char a)
{
	unsigned char i;
	unsigned char d=0;

	a|=0x80;	// to read

	DAC_nMS_L;			// nMS active=low

	for(i=0;i<8;i++)
	{
		if(a&0x80)
		{
			DAC_MDI_H;
		}
		else
		{
			DAC_MDI_L;
		}
		DAC_MC_H;
		a<<=1;
		DAC_MC_L;
	}

	for(i=0;i<8;i++)
	{
		d<<=1;
		if(DAC_MDOI)
		{
			d|=0x01;
		}
		else
		{
			d&=0xfe;
		}
		DAC_MC_H;
		DAC_MC_L;
	}

	DAC_nMS_H;			// nMS active=low

	return d;

}
