//**********************
// D-Amp and DCDC control with Latch
//
// (C)Copyright 2012 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#include "config.h"
#include "type.h"
#include "gpio.h"
#include "Damp.h"
#include "OLED.h"
#include "volume.h"
#include "remocon.h"
#include "timer32.h"
#include "DAC.h"

extern volatile unsigned char Buf[64];
extern volatile unsigned char flag;


volatile unsigned char VIRTUAL_flag=0;	// Virtual Power
volatile unsigned char AMP_flag=0;
volatile unsigned char POWER_flag=0;	// POWER_SAVE 0:OFF,1:ON
volatile unsigned char CSD_L_flag=CSD_OFF;
volatile unsigned char CSD_R_flag=CSD_OFF;
volatile unsigned char SELECT_flag=0;
volatile unsigned char USB_POWER_status=0;
volatile unsigned char STANDBY_status=0;
volatile unsigned char CSD_L_status=0;
volatile unsigned char CSD_R_status=0;

void InitDamp()
{

	/* Enable AHB clock to the GPIO domain. */
//	SCB_SYSAHBCLKCTRL |= (SCB_SYSAHBCLKCTRL_GPIO);

	/* Initialize port pins */
	Damp_STATUS_OUT;	// set output
	Damp_STATUS_H;		// high inactive
	Damp_POWER_OUT;		// set output
	Damp_POWER_H;		// high inactive
	Damp_DB0_OUT;		// set output
	Damp_DB1_OUT;		// set output
	Damp_DB2_OUT;		// set output
	Damp_DB3_OUT;		// set output

	Damp_DB0_L;			// POWER_SAVE 0:OFF,1:ON
	Damp_DB1_L;			// SD_L 0:ON,1:OFF
	Damp_DB2_L;			// SD_R 0:ON,1:OFF
	Damp_DB3_L;			// SELECTOR 0:OFF,1:ON
	Damp_POWER_L;		// low pulse to latch
	Damp_POWER_H;

	Damp_DB0_IN;		// set input
	Damp_DB1_IN;		// set input
	Damp_DB2_IN;		// set input
	Damp_DB3_IN;		// set input

	// Self Power Up
	while(STANDBY_status==0)
	{
		POWER_flag=1;
		CSD_L_flag=CSD_OFF;
		CSD_R_flag=CSD_OFF;
		SELECT_flag=0;
		SetDamp_Status();
		GetDamp_Status();
	}

}


void GetDamp_Status()
{

	Damp_DB0_IN;		// set input
	Damp_DB1_IN;		// set input
	Damp_DB2_IN;		// set input
	Damp_DB3_IN;		// set input
	Damp_STATUS_L;		// low pulse to latch
	STANDBY_status=Damp_DB0I;	// 0x01 or 0x00
	CSD_L_status=Damp_DB1I ? 1 : 0;		// 0x02 or 0x00
	CSD_R_status=Damp_DB2I ? 1 : 0;		// 0x04 or 0x00
	USB_POWER_status=Damp_DB3I;	// 0x08 or 0x00
	Damp_STATUS_H;

}


void SetDamp_Status()
{

	Damp_DB0_OUT;		// set output
	Damp_DB1_OUT;		// set output
	Damp_DB2_OUT;		// set output
	Damp_DB3_OUT;		// set output

	if(POWER_flag==0){
		Damp_DB0_L;			// POWER_SAVE 0:OFF,1:ON
	}else{
		Damp_DB0_H;			// POWER_SAVE 0:OFF,1:ON
	}
	if(CSD_L_flag==0){
		Damp_DB1_L;			// SD_L 0:ON,1:OFF
	}else{
		Damp_DB1_H;			// SD_L 0:ON,1:OFF
	}
	if(CSD_R_flag==0){
		Damp_DB2_L;			// SD_R 0:ON,1:OFF
	}else{
		Damp_DB2_H;			// SD_R 0:ON,1:OFF
	}
	if(SELECT_flag==0){
		Damp_DB3_L;			// SELECTOR 0:OFF,1:ON
	}else{
		Damp_DB3_H;			// SELECTOR 0:OFF,1:ON
	}
	Damp_POWER_L;		// low pulse to latch
	Damp_POWER_H;

	Damp_DB0_IN;		// set input
	Damp_DB1_IN;		// set input
	Damp_DB2_IN;		// set input
	Damp_DB3_IN;		// set input

}


void PrintSpeaker()
{
	if(AMP_flag)
	{
		PrintFont(0,104,"      ",BLUE, 1);
	}
	else
	{
		PrintFont(0,104,"PHONES",BLUE, 1);
	}
}


void DampPower()
{

	if(VIRTUAL_flag==0)
	{

		VIRTUAL_flag=1;
		AMP_flag=1;
		mute=0;
		WriteVolume(volume,volume);
		WriteDAC(RESISTER18, 0x50);	// DAC mute off
		// amp on
		CSD_L_flag=CSD_ON;
		CSD_R_flag=CSD_ON;
		SetDamp_Status();
		OLED_POWER_flag=1;
		PowerOnOLED();
		// Timer32-0
		timer32Init(0, TIMER32_CCLK_100MS);
		timer32Enable(0);
		timer32_0_counter=0;	// start

	}
	else
	{

		VIRTUAL_flag=0;
		AMP_flag=0;
		mute=1;
		WriteVolume(0x00,0x00);
		WriteDAC(RESISTER18, 0x51);	// DAC mute on
		// amp off
		CSD_L_flag=CSD_OFF;
		CSD_R_flag=CSD_OFF;
		SetDamp_Status();
		OLED_POWER_flag=0;
		PowerOffOLED();
		timer32Disable(0);

	}
	// tell power via USB
	Buf[2]=VIRTUAL_flag;
	Buf[1]=0;
	Buf[0]=POWER_KEY;	// POWER_KEY
	flag=1;

}


void Select()
{

	if(VIRTUAL_flag==0)
	{
		return;
	}

	if(SELECT_flag==0)
	{
		SELECT_flag=1;
		SetDamp_Status();
	}
	else
	{
		SELECT_flag=0;
		SetDamp_Status();
	}
	// tell select via USB
	Buf[2]=SELECT_flag;
	Buf[1]=0;
	Buf[0]=SELECT_KEY;	// SELECT_KEY
	flag=1;

}


void Phones()
{

	if(VIRTUAL_flag==0)
	{
		return;
	}

	if(AMP_flag==0)
	{

		AMP_flag=1;
		// amp on
		CSD_L_flag=CSD_ON;
		CSD_R_flag=CSD_ON;
		SetDamp_Status();
		timer32_0_counter=0;	// start
		PrintSpeaker();

	}
	else
	{

		AMP_flag=0;
		// amp off
		CSD_L_flag=CSD_OFF;
		CSD_R_flag=CSD_OFF;
		SetDamp_Status();
		PrintSpeaker();

	}
	// reset for volume with PHONE
	if(mute==0)
	{
		WriteVolume(volume,volume);
	}

	// tell phones via USB
	Buf[2]=AMP_flag;
	Buf[1]=0;
	Buf[0]=PHONES_KEY;	// PHONES_KEY
	flag=1;
}


void ScanDamp()
{
	GetDamp_Status();

	// power on 500ms delay
	if(CSD_L_status && CSD_L_flag==CSD_ON && timer32_0_counter>5){
		// error
		CSD_L_flag=CSD_OFF;	// stop speaker
		SetDamp_Status();
		PrintFont(0,64,"L:error",RED, 1);
		// volume minimum
		volume = 0x00;
		WriteVolume(volume,volume);
		PrintVolume();
		// tell CSD_L via USB
		Buf[2]=CSD_L_flag;
		Buf[1]=0;
		Buf[0]=CSD_L_KEY;	// CSD_L error
		flag=1;
	}
	if(CSD_R_status && CSD_R_flag==CSD_ON && timer32_0_counter>5){
		// error
		CSD_R_flag=CSD_OFF;	// stop speaker
		SetDamp_Status();
		PrintFont(0,56,"R:error",RED, 1);
		// volume minimum
		volume = 0x00;
		WriteVolume(volume,volume);
		PrintVolume();
		// tell CSD_R via USB
		Buf[2]=CSD_R_flag;
		Buf[1]=0;
		Buf[0]=CSD_R_KEY;	// CSD_R error
		flag=1;
	}
	if(STANDBY_status==0){
		// to check power down for emergency
		CSD_L_flag=CSD_OFF;	// stop speaker
		CSD_R_flag=CSD_OFF;	// stop speaker
		SetDamp_Status();
		VIRTUAL_flag=0;
		mute=1;
		WriteVolume(0x00,0x00);
		AMP_flag=0;
		OLED_POWER_flag=0;
		PowerOffOLED();
	}
	if(USB_POWER_status){
		// ToDo
	}
}

