//**********************
// FPGA control with I2C
//
// (C)Copyright 2012 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#include "type.h"
#include "gpio.h"
#include "i2c.h"
#include "FPGA.h"
#include "Damp.h"
#include "OLED.h"
#include "remocon.h"

extern volatile unsigned char Buf[64];
extern volatile unsigned char flag;

volatile unsigned char FPGA_status=0;
volatile unsigned char FPGA_fs=0xFF;


void InitFPGA()
{

	/* Enable AHB clock to the GPIO domain. */
	SCB_SYSAHBCLKCTRL |= (SCB_SYSAHBCLKCTRL_GPIO);

	/* Initialize port pins for SPI */
	FPGA_INT_IN;		// input
	// for SPI

	/* Initialize I2C */
	I2CInit( I2CMASTER );

	// Init for S/PDIF
	while(GetAK4118(0x00)!=0x2B)
	{
		GetAK4118(0x06);	// error clear
		GetAK4118(0x26);	// error clear
		PutAK4118(0x00,0x2B);
		PutAK4118(0x01,0x50);
		PutAK4118(0x02,0x00);
		PutAK4118(0x03,0xC0);
	}

	// Mask UNLCK
//	PutAK4118(0x04,0xFE);
//	GetAK4118(0x06);

}


void PrintFPGA_Status()
{
	if(SELECT_flag)
	{
		// Analog
		PrintFont2(10*6,13*8,"I:ANALOG",WHITE);
	}
	else
	{
		if(FPGA_status==0x80)
		{
			// USB audio
			PrintFont2(10*6,13*8,"I:USB   ",WHITE);
		}
		else
		{
			// S/PDIF
			PrintFont2(10*6,13*8,"I:S/PDIF",WHITE);
		}
	}

}

void GetFPGA_Status()
{

	static unsigned char prev_SELECT_flag;
	static unsigned char prev_FPGA_status;

	I2CWriteLength = 3;
	I2CReadLength = 1;
	I2CMasterBuffer[0] = FPGA_ADDR;
	I2CMasterBuffer[1] = 0x01;		/* high address */
	I2CMasterBuffer[2] = 0x00;		/* low address */
	I2CMasterBuffer[3] = FPGA_ADDR | RD_BIT;
	I2CEngine();

	FPGA_status = I2CSlaveBuffer[0];
	if(prev_SELECT_flag != SELECT_flag || prev_FPGA_status != FPGA_status)
	{
		PrintFPGA_Status();
		Buf[2]=FPGA_status;
		Buf[1]=0;
		Buf[0]=FPGA_KEY;	// FPGA status
		flag=1;
	}
	prev_SELECT_flag = SELECT_flag;
	prev_FPGA_status = FPGA_status;

}


void PrintFPGA_fs()
{

	switch(FPGA_fs)
	{
		case 0xFF:
			// **.*KHz
			PrintFont2(10*6,10*8," **.*KHz",CYAN);
			break;
		case 0:
			// 44.1KHz
			PrintFont2(10*6,10*8," 44.1KHz",CYAN);
			break;
		case 1:
		case 0x20:
			// 48KHz
			PrintFont2(10*6,10*8," 48.0KHz",CYAN);
			break;
		case 2:
		case 0x80:
			// 88.2KHz
			PrintFont2(10*6,10*8," 88.2KHz",CYAN);
			break;
		case 3:
		case 0xA0:
			// 96KHz
			PrintFont2(10*6,10*8," 96.0KHz",CYAN);
			break;
		case 4:
		case 0xC0:
			// 176.4KHz
			PrintFont2(10*6,10*8,"176.4KHz",CYAN);
			break;
		case 5:
		case 0xE0:
			// 192KHz
			PrintFont2(10*6,10*8,"192.0KHz",CYAN);
			break;
	}
}


void GetFPGA_fs()
{

	static unsigned char prev_FPGA_fs;

	if(FPGA_status==0x80)
	{
		// USB audio
		I2CWriteLength = 3;
		I2CReadLength = 1;
		I2CMasterBuffer[0] = FPGA_ADDR;
		I2CMasterBuffer[1] = 0x01;		/* high address */
		I2CMasterBuffer[2] = 0x01;		/* low address */
		I2CMasterBuffer[3] = FPGA_ADDR | RD_BIT;
		I2CEngine();
		FPGA_fs=I2CSlaveBuffer[0];
	}
	else
	{
		// S/PDIF
		if((GetAK4118(0x06) & 0x10) == 0x10)
		{
			FPGA_fs=0xFF;
		}
		else
		{
			FPGA_fs=GetAK4118(0x07) & 0xf0;
		}
	}

	if(prev_FPGA_fs != FPGA_fs)
	{
		PrintFPGA_fs();
		Buf[2]=FPGA_fs;
		Buf[1]=0;
		Buf[0]=FS_KEY;	// FPGA fs
		flag=1;
	}
	prev_FPGA_fs = FPGA_fs;

}


void ScanFPGA()
{

	if(OLED_POWER_flag==0)
	{
		return;
	}

	// caution: not cleared FPGA_INT
	GetFPGA_Status();
	GetFPGA_fs();

}



unsigned char GetAK4118(unsigned char a)
{

	I2CWriteLength = 3;
	I2CReadLength = 1;
	I2CMasterBuffer[0] = FPGA_ADDR;
	I2CMasterBuffer[1] = 0x00;		// high address
	I2CMasterBuffer[2] = a;			// low address 0x00 - 0x28 for AK4118
	I2CMasterBuffer[3] = FPGA_ADDR | RD_BIT;
	I2CEngine();

	return I2CSlaveBuffer[0];

}


void PutAK4118(unsigned char a, unsigned char d)
{

	I2CWriteLength = 4;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = FPGA_ADDR;
	I2CMasterBuffer[1] = 0x00;		// high address
	I2CMasterBuffer[2] = a;			// low address 0x00 - 0x28 for AK4118
	I2CMasterBuffer[3] = d;
	I2CEngine();

}

