//**********************
// OLED control with bus
//
// 160x132 dot
//
// (0,131)-(159,131)
// |               |
// (0,  0)-(159,  0)
// (C)Copyright 2012 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#include "config.h"
#include "type.h"
#include "gpio.h"
#include "OLED.h"
#include "font.h"
#include "timer32.h"
#include "timer16.h"
#include "volume.h"
#include "FPGA.h"
#include "Damp.h"
#include "remocon.h"

extern volatile unsigned char Buf[64];
extern volatile unsigned char flag;

volatile unsigned char OLED_POWER_flag=0;


static inline void GPIOSetPortL( uint32_t portNum, uint32_t Val )
{
	LPC_GPIO[portNum]->MASKED_ACCESS[0xff] = Val;
}

static inline void GPIOSetOutDirL( uint32_t portNum, uint32_t dir )
{
		LPC_GPIO[portNum]->DIR |= dir;
}

static inline void GPIOSetInDirL( uint32_t portNum, uint32_t dir )
{
		LPC_GPIO[portNum]->DIR &= (~dir);
}


void OutOLED(unsigned char a, unsigned char d)
{
	if(d)
	{
		OLED_DC_H;			// data
	}
	else
	{
		OLED_DC_L;			// command
	}
	OLED_CS_L;			// chip select
	OLED_WR_L;			// to write
	GPIOSetPortL(PORT2, a);
	OLED_WR_H;
	OLED_CS_H;

}

void InitOLED()
{

	/* Initialize port pins */
	OLED_CS_OUT;		// set output
	OLED_RES_OUT;		// set output
	OLED_DC_OUT;		// set output
	OLED_WR_OUT;		// set output
	OLED_RD_OUT;		// set output
	OLED_POWER_OUT;		// set output
	GPIOSetInDirL(PORT2, 0xFF);	// set input

	OLED_POWER_L;		// POWER 0:OFF,1:ON
	OLED_CS_H;			// CS
	OLED_RES_H;			// RES active=low pulse
	OLED_DC_L;			// DC 0:command,1:data
	OLED_WR_H;			// WR 0:enable,1:disable
	OLED_RD_H;			// RD 0:enable,1:disable

	OLED_POWER_flag=0;	// default off

}


void PowerOnOLED()
{

	GPIOSetOutDirL(PORT2, 0xFF);	// set output

	OLED_POWER_H;		// POWER 0:OFF,1:ON
	//wait 100us
	WaitNms(1);
	OLED_RES_L;			// reset 3us
	OLED_RES_H;

	// send display on
	OutOLED(0xAF, OLED_command);	// display on

	// send driver remap and color depth
	OutOLED(0xA0, OLED_command);	// driver remap and color depth
//	OutOLED(0x60, OLED_data);		// data= 01100000 for 65K color
//	OutOLED(0x70, OLED_data);		// data= 01110000 for 65K color
	OutOLED(0xA0, OLED_data);		// data= 10100000 for 262K color

	// send clear window
	OutOLED(0x25, OLED_command);	// clear window
	OutOLED(0, OLED_data);			// column address of start
	OutOLED(0, OLED_data);			// row address of start
	OutOLED(159, OLED_data);		// column address of end
	OutOLED(131, OLED_data);		// row address of end
	// wait ...
	WaitNms(2);

	GPIOSetInDirL(PORT2, 0xFF);	// set input

	// initial display
	PrintFont(0,36,"L:",WHITE, 1);
	PrintFont(0,12,"R:",WHITE, 1);

	GPIOSetOutDirL(PORT2, 0xFF);	// set output

	// Draw lines
	unsigned char i;
	for(i=0;i<10;i++)
	{
		OutOLED(0x21, OLED_command);	// draw line
		OutOLED(i*16+14, OLED_data);	// column start
		OutOLED(26, OLED_data);			// row start
		OutOLED(i*16+14, OLED_data);	// column end
		OutOLED(30, OLED_data);			// row end
		OutOLED(0x3F, OLED_data);		// color C of the line
		OutOLED(0x3F, OLED_data);		// color B of the line
		OutOLED(0x3F, OLED_data);		// color A of the line
	}

	GPIOSetInDirL(PORT2, 0xFF);	// set input

	PrintFPGA_Status();
	PrintFPGA_fs();
	PrintVolume();
	PrintSpeaker();

}


void PowerOffOLED()
{

	GPIOSetOutDirL(PORT2, 0xFF);	// set output

	// send display off
	OutOLED(0xAE, OLED_command);	// display off

	// wait 100ms
	OLED_POWER_L;		// POWER 0:OFF,1:ON

	GPIOSetInDirL(PORT2, 0xFF);	// set input

}


void PowerOLED()
{

	if(VIRTUAL_flag==0)
	{
		return;
	}

	if(OLED_POWER_flag==0)
	{
		OLED_POWER_flag=1;
		PowerOnOLED();
	}
	else
	{
		OLED_POWER_flag=0;
		PowerOffOLED();
	}
	// tell OLED via USB
	Buf[2]=OLED_POWER_flag;
	Buf[1]=0;
	Buf[0]=OLED_KEY;	// OLED_KEY
	flag=1;

}


void SetFont(unsigned char x, unsigned char y, unsigned char a, unsigned int c, unsigned char f)
{

	unsigned char i,j,k,l;
	unsigned char fx,fy;	// font size

	if(f==1)
	{
		fx=5;
		fy=7;
	}
	else
	{
		fx=11;
		fy=15;
		f=2;
	}

	GPIOSetOutDirL(PORT2, 0xFF);	// set output

	// set area
	OutOLED(0x15, OLED_command);	// set column start end
	OutOLED(0+x, OLED_data);		// column start
	OutOLED(fx+x, OLED_data);		// column end
	OutOLED(0x75, OLED_command);	// set row start end
	OutOLED(0+y, OLED_data);		// row start
	OutOLED(fy+y, OLED_data);		// row end

	// set font
	OutOLED(0x5C, OLED_command);	// write ram
	for(i=0;i<8;i++)
	{
		for(l=0;l<f;l++)
		{

		for(j=0;j<5;j++)
		{
			if((Font[a-0x20][j])&(1<<i))
			{
				for(k=0;k<f;k++)
				{
					OutOLED((c>>16)&0xFF, OLED_data);	// first color for R
					OutOLED((c>>8)&0xFF, OLED_data);	// second color for G
					OutOLED(c&0xFF, OLED_data);			// third color for B
				}
			}
			else
			{
				for(k=0;k<f;k++)
				{
					OutOLED(0x00, OLED_data);
					OutOLED(0x00, OLED_data);
					OutOLED(0x00, OLED_data);
				}

			}
		}

		// space
		for(k=0;k<f;k++)
		{
			OutOLED(0x00, OLED_data);
			OutOLED(0x00, OLED_data);
			OutOLED(0x00, OLED_data);
		}

		}

	}
	GPIOSetInDirL(PORT2, 0xFF);	// set input

	// wait ...
//	Wait1ms(1);


}



void PrintFont(unsigned char x, unsigned char y, char *a, unsigned int c, unsigned char f)
{
	unsigned char xx;

	if(OLED_POWER_flag==0)
	{
		return;
	}

	xx=x;
	while(*a!=0)
	{
		SetFont(xx, y, *a++, c, f);
		xx+=(6*f);
	}

}


void SetFont2(unsigned char x, unsigned char y, unsigned char a, unsigned int c)
{

	unsigned char i,j,k,m;
	unsigned char fx,fy;	// font size

	fx=11;
	fy=15;

	GPIOSetOutDirL(PORT2, 0xFF);	// set output

	// set area
	OutOLED(0x15, OLED_command);	// set column start end
	OutOLED(0+x, OLED_data);		// column start
	OutOLED(fx+x, OLED_data);		// column end
	OutOLED(0x75, OLED_command);	// set row start end
	OutOLED(0+y, OLED_data);		// row start
	OutOLED(fy+y, OLED_data);		// row end

	// set font
	OutOLED(0x5C, OLED_command);	// write ram
	m=11;
	for(k=0;k<2;k++)
	{

		for(i=0;i<8;i++)
		{

			for(j=0;j<11;j++)
			{
				if((Font2[a-0x20][j+m])&(1<<i))
				{
					OutOLED((c>>16)&0xFF, OLED_data);	// first color for R
					OutOLED((c>>8)&0xFF, OLED_data);	// second color for G
					OutOLED(c&0xFF, OLED_data);			// third color for B
				}
				else
				{
					OutOLED(0x00, OLED_data);
					OutOLED(0x00, OLED_data);
					OutOLED(0x00, OLED_data);
				}
			}

			// space
			OutOLED(0x00, OLED_data);
			OutOLED(0x00, OLED_data);
			OutOLED(0x00, OLED_data);

		}
		m=0;

	}

	GPIOSetInDirL(PORT2, 0xFF);	// set input

	// wait ...
//	Wait1ms(1);


}


void PrintFont2(unsigned char x, unsigned char y, char *a, unsigned int c)
{
	unsigned char xx;

	if(OLED_POWER_flag==0)
	{
		return;
	}

	xx=x;
	while(*a!=0)
	{
		SetFont2(xx, y, *a++, c);
		xx+=12;
	}

}


// Level = -94dB to 0dB step -2
void LevelMeter(int u, int p, unsigned char LR)
{

	int i;
	unsigned char B,G,R;
	unsigned char y1,y2;

	if(OLED_POWER_flag==0)
	{
		return;
	}

	if(u < -94)u=-94;
	if(p < -94)p=-94;

	if(LR==0)
	{
		y1=36;
		y2=44;
	}
	else
	{
		y1=11;
		y2=19;
	}

	GPIOSetOutDirL(PORT2, 0xFF);	// set output

	for(i=-47;i<=0;i++)
	{
		// BGR
		B=0x00;
		G=0x3F-(47+i);
		R=(47+i);
		if(u/2<i)		// UV
		{
			B=0x00;
			G=0x00;
			R=0x00;
		}
/* peak
		if(p/2==i)	// PM
		{
			B=0x00;
			G=0x00;
			R=0x3F;
		}
*/
		// Draw rectangle
		OutOLED(0x22, OLED_command);	// draw rectangle
		OutOLED((47+i)*3+14, OLED_data);		// column start
		OutOLED(y1, OLED_data);			// row start
		OutOLED((47+i)*3+15, OLED_data);		// column end
		OutOLED(y2, OLED_data);			// row end
		OutOLED(B, OLED_data);			// color C of the line
		OutOLED(G, OLED_data);			// color B of the line
		OutOLED(R, OLED_data);			// color A of the line
		OutOLED(B, OLED_data);			// color C of the fill area
		OutOLED(G, OLED_data);			// color B of the fill area
		OutOLED(R, OLED_data);			// color A of the fill area
	}

	GPIOSetInDirL(PORT2, 0xFF);	// set input

}

