//**********************
// adc for LPC1343
//
// (C)Copyright 2011 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#include "adc.h"
#include "remocon.h"
#include "OLED.h"
#include "timer32.h"
#include "volume.h"
#include <math.h>

extern volatile unsigned char Buf[64];
extern volatile unsigned char flag;

// use PMmeter circuit for level meter
// If use this option, remove comment out #define USE_PM_FOR_LEVEL_METER
#define USE_PM_FOR_LEVEL_METER

// use modified VUmeter circuit(gain=x10 & tc=100ms )
// remove C102/C152, change R107/R157 to 0ohm,
// change R108/R158 to 1Mohm, change C103/C153 to 0.1uf.
// If use this option, remove comment out #define USE_VM_CUSTOM_FOR_LEVEL_METER
//#define USE_VM_CUSTOM_FOR_LEVEL_METER

//**********************
// VU-R
// get adc0 with 10bits
//**********************
unsigned int GetVU_R()
{

  unsigned int regVal, adcData;

  /* Deselect all channels */
  ADC_AD0CR &= ~ADC_AD0CR_SEL_MASK;

  /* Start converting now on the appropriate channel */
  ADC_AD0CR |= ADC_AD0CR_START_STARTNOW | ADC_AD0CR_SEL_AD0;

  /* wait until end of A/D convert */
  do{
    regVal = *(volatile unsigned int *)(ADC_AD0DR0);
  }while ( !(regVal & ADC_DR_DONE) );

  /* stop ADC */
  ADC_AD0CR &= ~ADC_AD0CR_START_MASK;

  /* return 0 if an overrun occurred */
  if ( regVal & ADC_DR_OVERRUN )
  {
    return (0);
  }

  /* return conversion results */
  adcData = (regVal >> 6) & 0x3FF;
  return (adcData);
}


//**********************
// PM-R
// get adc1 with 10bits
//**********************
unsigned int GetPM_R()
{

  unsigned int regVal, adcData;

  /* Deselect all channels */
  ADC_AD0CR &= ~ADC_AD0CR_SEL_MASK;

  /* Start converting now on the appropriate channel */
  ADC_AD0CR |= ADC_AD0CR_START_STARTNOW | ADC_AD0CR_SEL_AD1;

  /* wait until end of A/D convert */
  do{
    regVal = *(volatile unsigned int *)(ADC_AD0DR1);
  }while ( !(regVal & ADC_DR_DONE) );

  /* stop ADC */
  ADC_AD0CR &= ~ADC_AD0CR_START_MASK;

  /* return 0 if an overrun occurred */
  if ( regVal & ADC_DR_OVERRUN )
  {
    return (0);
  }

  /* return conversion results */
  adcData = (regVal >> 6) & 0x3FF;
  return (adcData);
}


//**********************
// VU-L
// get adc2 with 10bits
//**********************
unsigned int GetVU_L()
{

  unsigned int regVal, adcData;

  /* Deselect all channels */
  ADC_AD0CR &= ~ADC_AD0CR_SEL_MASK;

  /* Start converting now on the appropriate channel */
  ADC_AD0CR |= ADC_AD0CR_START_STARTNOW | ADC_AD0CR_SEL_AD2;

  /* wait until end of A/D convert */
  do{
    regVal = *(volatile unsigned int *)(ADC_AD0DR2);
  }while ( !(regVal & ADC_DR_DONE) );

  /* stop ADC */
  ADC_AD0CR &= ~ADC_AD0CR_START_MASK;

  /* return 0 if an overrun occurred */
  if ( regVal & ADC_DR_OVERRUN )
  {
    return (0);
  }

  /* return conversion results */
  adcData = (regVal >> 6) & 0x3FF;
  return (adcData);
}


//**********************
// PM-L
// get adc3 with 10bits
//**********************
unsigned int GetPM_L()
{

  unsigned int regVal, adcData;

  /* Deselect all channels */
  ADC_AD0CR &= ~ADC_AD0CR_SEL_MASK;

  /* Start converting now on the appropriate channel */
  ADC_AD0CR |= ADC_AD0CR_START_STARTNOW | ADC_AD0CR_SEL_AD3;

  /* wait until end of A/D convert */
  do{
    regVal = *(volatile unsigned int *)(ADC_AD0DR3);
  }while ( !(regVal & ADC_DR_DONE) );

  /* stop ADC */
  ADC_AD0CR &= ~ADC_AD0CR_START_MASK;

  /* return 0 if an overrun occurred */
  if ( regVal & ADC_DR_OVERRUN )
  {
    return (0);
  }

  /* return conversion results */
  adcData = (regVal >> 6) & 0x3FF;
  return (adcData);
}


//**********************
// Initialize adc
//**********************
void InitAdc()
{
  /* Disable Power down bit to the ADC block. */
  SCB_PDRUNCFG &= ~(SCB_PDRUNCFG_ADC);

  /* Enable AHB clock to the ADC. */
  SCB_SYSAHBCLKCTRL |= (SCB_SYSAHBCLKCTRL_ADC);

  /* Digital pins need to have the 'analog' bit set in addition
     to changing their pin function */

  /* Set AD0 to analog input */
  IOCON_JTAG_TDI_PIO0_11 &= ~(IOCON_JTAG_TDI_PIO0_11_ADMODE_MASK |
                              IOCON_JTAG_TDI_PIO0_11_FUNC_MASK |
                              IOCON_JTAG_TDI_PIO0_11_MODE_MASK);
  IOCON_JTAG_TDI_PIO0_11 |=  (IOCON_JTAG_TDI_PIO0_11_FUNC_AD0 &
                              IOCON_JTAG_TDI_PIO0_11_ADMODE_ANALOG);

  /* Set AD1 to analog input */
  IOCON_JTAG_TMS_PIO1_0 &= ~(IOCON_JTAG_TMS_PIO1_0_ADMODE_MASK |
                              IOCON_JTAG_TMS_PIO1_0_FUNC_MASK |
                              IOCON_JTAG_TMS_PIO1_0_MODE_MASK);
  IOCON_JTAG_TMS_PIO1_0 |=  (IOCON_JTAG_TMS_PIO1_0_FUNC_AD1 &
                              IOCON_JTAG_TMS_PIO1_0_ADMODE_ANALOG);

  /* Set AD2 to analog input */
  IOCON_JTAG_TDO_PIO1_1 &= ~(IOCON_JTAG_TDO_PIO1_1_ADMODE_MASK |
                              IOCON_JTAG_TDO_PIO1_1_FUNC_MASK |
                              IOCON_JTAG_TDO_PIO1_1_MODE_MASK);
  IOCON_JTAG_TDO_PIO1_1 |=  (IOCON_JTAG_TDO_PIO1_1_FUNC_AD2 &
                              IOCON_JTAG_TDO_PIO1_1_ADMODE_ANALOG);

  /* Set AD3 to analog input */
  IOCON_JTAG_nTRST_PIO1_2 &= ~(IOCON_JTAG_nTRST_PIO1_2_ADMODE_MASK |
                              IOCON_JTAG_nTRST_PIO1_2_FUNC_MASK |
                              IOCON_JTAG_nTRST_PIO1_2_MODE_MASK);
  IOCON_JTAG_nTRST_PIO1_2 |=  (IOCON_JTAG_nTRST_PIO1_2_FUNC_AD3 &
                              IOCON_JTAG_nTRST_PIO1_2_ADMODE_ANALOG);


  /* Note that in SW mode only one channel can be selected at a time (AD0 in this case)
     To select multiple channels, ADC_AD0CR_BURST_HWSCANMODE must be used */
  ADC_AD0CR = (ADC_AD0CR_SEL_AD0 |                     /* SEL=1,select channel 0 on ADC0 */
              (((CFG_CPU_CCLK / SCB_SYSAHBCLKDIV) / 1000000 - 1 ) << 8) |   /* CLKDIV = Fpclk / 1000000 - 1 */
              ADC_AD0CR_BURST_SWMODE |                 /* BURST = 0, no BURST, software controlled */
              ADC_AD0CR_CLKS_10BITS |                  /* CLKS = 0, 11 clocks/10 bits */
              ADC_AD0CR_START_NOSTART |                /* START = 0 A/D conversion stops */
              ADC_AD0CR_EDGE_RISING);                  /* EDGE = 0 (CAP/MAT signal falling, trigger A/D conversion) */ 

}


void ScanAdc()
{
	int VU_R,PM_R,VU_L,PM_L;
	int offset;

//	offset = (volume/2-96)+20;	// Damp gain=20dB
	offset = (volume/2-96);

	// analog 0dB = 1Vrms, Digital 0dB = 1Vrms
	// V=AD/1023*3.3
	// dB=20*log10(V)
	// -128dB through +127dB
	#ifndef USE_PM_FOR_LEVEL_METER
		#ifndef USE_VM_CUSTOM_FOR_LEVEL_METER
			// normal
			VU_L = 20*log10((double)GetVU_L()/1023*3.3)+offset;
			PM_L = 20*log10((double)GetPM_L()/1023*3.3)+offset;
			VU_R = 20*log10((double)GetVU_R()/1023*3.3)+offset;
			PM_R = 20*log10((double)GetPM_R()/1023*3.3)+offset;
		#else
			// use modified VUmeter circuit(gain=x10 & tc=100ms )
			// remove C102/C152, change R107/R157 to 0ohm,
			// change R108/R158 to 1Mohm, change C103/C153 to 0.1uf.
			VU_L = 20*log10((double)GetVU_L()/10230*3.3)+offset;
			PM_L = 20*log10((double)GetPM_L()/10230*3.3)+offset;
			VU_R = 20*log10((double)GetVU_R()/10230*3.3)+offset;
			PM_R = 20*log10((double)GetPM_R()/10230*3.3)+offset;
		#endif
	#else
		// use PMmeter circuit for level meter
		VU_L = 20*log10((double)GetPM_L()/1023*3.3)+offset;
		PM_L = 20*log10((double)GetPM_L()/1023*3.3)+offset;
		VU_R = 20*log10((double)GetPM_R()/1023*3.3)+offset;
		PM_R = 20*log10((double)GetPM_R()/1023*3.3)+offset;
	#endif
	if(VU_L<-128)VU_L=-128;
	if(VU_R<-128)VU_R=-128;

	// dB   -94dB to 0dB step -2
	LevelMeter(VU_L, PM_L, 0);
	LevelMeter(VU_R, PM_R, 1);

	if(flag==0){
		// tell level via USB
		Buf[0]=LEVEL_KEY;	// LEVEL
		Buf[1]=VU_L&0xff;
		Buf[2]=PM_L&0xff;
		Buf[3]=VU_R&0xff;
		Buf[4]=PM_R&0xff;
		flag=1;
	}
}



