//**********************
// Timer16
//
// (C)Copyright 2012 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#include "LPC13xx.h"
#include "timer16.h"

// unsigned int = 32bit
volatile unsigned int timer16_0_counter = 0;
volatile unsigned int timer16_1_counter = 0;


//**********************
// wait
// caution: interrupt conflict
//**********************
void WaitNms(unsigned int n)
{

	timer16Delay(1,n);
}

//**********************
// Delay ticks
//　max count: 0xFFFF / 48 = 1365us
//**********************
void timer16Delay(unsigned char timerNum, unsigned int delay)
{

	if (timerNum == 0)
	{
		timer16_0_counter=0;
		while (timer16_0_counter <= delay);
    }

	if (timerNum == 1)
	{
		timer16_1_counter=0;
		while (timer16_1_counter <= delay);
	}

}


//**********************
// Interrupt handler for 16-bit timer 0
//**********************
void TIMER16_0_IRQHandler(void)
{
  /* Clear the interrupt flag */
  TMR_TMR16B0IR = TMR_TMR16B0IR_MR0;

  timer16_0_counter++;
}


//**********************
// Interrupt handler for 16-bit timer 1
//**********************
void TIMER16_1_IRQHandler(void)
{
  /* Clear the interrupt flag */
  TMR_TMR16B1IR = TMR_TMR16B1IR_MR0;

  timer16_1_counter++;
}


//**********************
// Enables the specified timer
//**********************
void timer16Enable(unsigned char timerNum)
{
  if ( timerNum == 0 )
  {
    TMR_TMR16B0TCR = TMR_TMR16B0TCR_COUNTERENABLE_ENABLED;
  }

  if (timerNum == 1)
  {
    TMR_TMR16B1TCR = TMR_TMR16B1TCR_COUNTERENABLE_ENABLED;
  }

}


//**********************
// Disables the specified timer
//**********************
void timer16Disable(unsigned char timerNum)
{
  if ( timerNum == 0 )
  {
    TMR_TMR16B0TCR = TMR_TMR16B0TCR_COUNTERENABLE_DISABLED;
  }

  else if (timerNum == 1)
  {
    TMR_TMR16B1TCR = TMR_TMR16B1TCR_COUNTERENABLE_DISABLED;
  }

  return;
}


//**********************
// Initialises the specified timer
//**********************
void timer16Init(unsigned char timerNum, unsigned int timerInterval)
{

  if ( timerNum == 0 )
  {
    /* Enable the clock for CT16B0 */
    SCB_SYSAHBCLKCTRL |= (SCB_SYSAHBCLKCTRL_CT16B0);

    timer16_0_counter = 0;
    TMR_TMR16B0MR0 = timerInterval;

    /* Configure match control register to raise an interrupt and reset on MR0 */
    TMR_TMR16B0MCR = (TMR_TMR16B0MCR_MR0_INT_ENABLED | TMR_TMR16B0MCR_MR0_RESET_ENABLED);

    /* Enable the TIMER0 interrupt */
    NVIC_EnableIRQ(TIMER_16_0_IRQn);
  }

  if ( timerNum == 1 )
  {
    /* Enable the clock for CT16B1 */
    SCB_SYSAHBCLKCTRL |= (SCB_SYSAHBCLKCTRL_CT16B1);

    timer16_1_counter = 0;
    TMR_TMR16B1MR0 = timerInterval;

    /* Configure match control register to raise an interrupt and reset on MR0 */
    TMR_TMR16B1MCR = (TMR_TMR16B1MCR_MR0_INT_ENABLED | TMR_TMR16B1MCR_MR0_RESET_ENABLED);

    /* Enable the TIMER1 Interrupt */
    NVIC_EnableIRQ(TIMER_16_1_IRQn);
  }

}

