//**********************
// Timer32
//
// caution: Do not use Timer1 with usbhidrom
// (C)Copyright 2012 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#include "LPC13xx.h"
#include "timer32.h"
#include "FPGA.h"
#include "Damp.h"
#include "adc.h"
#include "remocon.h"


// unsigned int = 32bit
volatile unsigned int timer32_0_counter = 0;
volatile unsigned int timer32_1_counter = 0;
volatile unsigned char TIMER_flag = 0;
extern volatile unsigned char USB_flag;
extern volatile unsigned char dispatch_flag;
extern volatile unsigned char SCAN_flag;

//**********************
// wait
//**********************
void Wait100ms(unsigned int n)
{
	timer32Delay(0,n);
}

//**********************
// Delay ticks
//**********************
void timer32Delay(unsigned char timerNum, unsigned int delay)
{

	if (timerNum == 0)
	{
		timer32_0_counter=0;
		while (timer32_0_counter <= delay);
	}

	if (timerNum == 1)
	{
    	timer32_1_counter=0;
    	while (timer32_1_counter <= delay);
    }

}


//**********************
// Interrupt handler for 32-bit timer 0
//**********************
void TIMER32_0_IRQHandler(void)
{
  /* Clear the interrupt flag */
  TMR_TMR32B0IR = TMR_TMR32B0IR_MR0;

  timer32_0_counter++;
  // Confirm InitFPGA and InitDamp
  if(REMOCON_flag==0 && USB_flag==0 && SCAN_flag==0 && dispatch_flag==0)
  {
	  ScanAdc();
  }

  TIMER_flag=1;	// for ScanFPGA()

}

//**********************
// Interrupt handler for 32-bit timer 1
//**********************
void TIMER32_1_IRQHandler(void)
{
  /* Clear the interrupt flag */
  TMR_TMR32B1IR = TMR_TMR32B1IR_MR0;

  timer32_1_counter++;

}


//**********************
// Enables the specified timer
//**********************
void timer32Enable(unsigned char timerNum)
{
  if ( timerNum == 0 )
  {
    TMR_TMR32B0TCR = TMR_TMR32B0TCR_COUNTERENABLE_ENABLED;
  }

  if ( timerNum == 1)
  {
    TMR_TMR32B1TCR = TMR_TMR32B1TCR_COUNTERENABLE_ENABLED;
  }

}


//**********************
// Disables the specified timer
//**********************
void timer32Disable(unsigned char timerNum)
{
  if ( timerNum == 0 )
  {
    TMR_TMR32B0TCR = TMR_TMR32B0TCR_COUNTERENABLE_DISABLED;
  }

  if ( timerNum == 1)
  {
    TMR_TMR32B1TCR = TMR_TMR32B1TCR_COUNTERENABLE_DISABLED;
  }

}


//**********************
// Initialises the specified timer
//**********************
void timer32Init(unsigned char timerNum, unsigned int timerInterval)
{

  if ( timerNum == 0 )
  {
    /* Enable the clock for CT32B0 */
    SCB_SYSAHBCLKCTRL |= (SCB_SYSAHBCLKCTRL_CT32B0);

    timer32_0_counter = 0;
    TMR_TMR32B0MR0 = timerInterval;

    /* Configure match control register to raise an interrupt and reset on MR0 */
    TMR_TMR32B0MCR = (TMR_TMR32B0MCR_MR0_INT_ENABLED | TMR_TMR32B0MCR_MR0_RESET_ENABLED);

    /* Enable the TIMER0 interrupt */
    NVIC_EnableIRQ(TIMER_32_0_IRQn);
  }

  if ( timerNum == 1 )
  {
    /* Enable the clock for CT32B1 */
    SCB_SYSAHBCLKCTRL |= (SCB_SYSAHBCLKCTRL_CT32B1);

    timer32_1_counter = 0;
    TMR_TMR32B1MR0 = timerInterval;

    /* Configure match control register to raise an interrupt and reset on MR0 */
    TMR_TMR32B1MCR = (TMR_TMR32B1MCR_MR0_INT_ENABLED | TMR_TMR32B1MCR_MR0_RESET_ENABLED);

    /* Enable the TIMER1 Interrupt */
    NVIC_EnableIRQ(TIMER_32_1_IRQn);
  }

}

