//**********************
// LV 1.0 for LPC1343
// based on USB HID rom
//
// 48MHz clock, 20.83ns
// (C)Copyright 2012 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#include "LPC13xx.h"                        /* LPC13xx definitions */
#include "usb.h"
#include "usbdesc.h"
#include "gpio.h"
#include "rom_drivers.h"
#include "config.h"
#include "adc.h"
#include "remocon.h"
#include "Damp.h"
#include "OLED.h"
#include "FPGA.h"
#include "DAC.h"
#include "volume.h"
#include "EEPROM.h"
#include "timer32.h"

volatile unsigned char Buf[64];		// USB buffer
volatile unsigned char flag=0;		// to send out USB packets
volatile unsigned char dispatch_code=0;
volatile unsigned char dispatch_flag=0;
volatile unsigned char SCAN_flag=0;

//=Program Version info =========================
volatile unsigned char Ver[5] = "1.00r";
//===============================================

#define     EN_TIMER32_1    (1<<10)
#define     EN_IOCON        (1<<16)
#define     EN_USBREG       (1<<14)

USB_DEV_INFO DeviceInfo;
HID_DEVICE_INFO HidDevInfo;
ROM ** rom = (ROM **)0x1fff1ff8;


// priority control for printing
// REMOCON_flag > USB_flag > SCAN_flag > INTERRUPT
volatile unsigned char USB_flag = 0;

// system
#define SCB_PLLCTRL                               (*((unsigned int *) (0x40048008)))    // System PLL control
#define SCB_PLLSTAT                               (*((unsigned int *) (0x4004800C)))    // System PLL status
#define SCB_USBPLLCTRL                            (*((unsigned int *) (0x40048010)))    // USB PLL control
#define SCB_USBPLLSTAT                            (*((unsigned int *) (0x40048014)))    // USB PLL status
#define SCB_SYSOSCCTRL                            (*((unsigned int *) (0x40048020)))    // System oscillator control
#define SCB_DEVICEID                              (*((unsigned int *) (0x400483F4)))    // Device ID


void Jacket()
{

	// ToDo

	// system test
//	Buf[2]=(SCB_DEVICEID>>16) & 0xff;
//	Buf[1]=(SCB_DEVICEID>>24) & 0xff;
//	Buf[0]=0x41;
//	flag=1;

	// ADC test
//	static unsigned char i=16;
//	Buf[2]=ReadDAC(i);
//	Buf[1]=i;
//	Buf[0]=0x41;
//	flag=1;
//	++i;

	// EEPROM test
//	WriteEEPROM(0, 0x5A);
//	Buf[2]=ReadEEPROM(0);
//	Buf[1]=0;
//	Buf[0]=0x41;
//	flag=1;

	// DAC test
//	WriteDAC(RESISTER18, 0x51);
//	Buf[2]=ReadDAC(RESISTER18);
//	Buf[1]=0;
//	Buf[0]=0x41;
//	flag=1;

	// AK4118 test
//	Buf[2]=GetAK4118(6);
//	Buf[1]=GetAK4118(7);
//	Buf[0]=0x41;
//	flag=1;

}

void Status()
{

	Buf[0]=STATUS_KEY;
	Buf[1]=VIRTUAL_flag;
	Buf[2]=mute;
	Buf[3]=volume;
	Buf[4]=OLED_POWER_flag;
	Buf[5]=SELECT_flag;
	Buf[6]=AMP_flag;
	Buf[7]=FPGA_status;
	Buf[8]=FPGA_fs;
	flag=1;

}

void Version()
{
	Buf[0]=VERSION_KEY;
	Buf[1]=VIRTUAL_flag;
	Buf[2]=Ver[0];
	Buf[3]=Ver[1];
	Buf[4]=Ver[2];
	Buf[5]=Ver[3];
	Buf[6]=Ver[4];
	flag=1;

}

void Dispatch()
{
	dispatch_flag=1;
	switch(dispatch_code){
		case 0:
			break;
		case POWER_KEY:
			DampPower();
			break;
		case MUTE_KEY:
			MuteVolume();
			break;
		case OLED_KEY:
			PowerOLED();
			break;
		case UP_KEY:
			UpVolume();
			break;
		case DOWN_KEY:
			DownVolume();
			break;
		case JACKET_KEY:
			Jacket();
			break;
		case SELECT_KEY:
			Select();
			break;
		case PHONES_KEY:
			Phones();
			break;
		case STATUS_KEY:
			Status();
			break;
		case VERSION_KEY:
			Version();
			break;
	}
	dispatch_code=0;
	dispatch_flag=0;
}


/*
 *  Get HID Input Report -> InReport
 */
// LPC->PC
void GetInReport (uint8_t src[], uint32_t length)
{
	unsigned char i;

	/* Check the input port bits and set the bits of
	 * the "InReport" response for the PC.
	 */

	if(flag==1){
		for(i=0;i<64;i++){
			src[i]=Buf[i];
		}
		flag=0;
	}else{
		src[0]=0;
	}

}


/*
 *  Set HID Output Report <- OutReport
 */
// PC->LPC
void SetOutReport (uint8_t dst[], uint32_t length)
{
	unsigned char i;

	/* Check the bits of the "OutReport" data from the PC
	 * and set the output port status.
	 */

	USB_flag = 1;
	for(i=0;i<64;i++){
		Buf[i]=dst[i];
	}
	dispatch_code = Buf[0];
	USB_flag = 0;

}


int main (void)
{
	/* for delay loop */
	volatile int n;
// Code Red Red Suite and LPCXpresso by Code Red both call SystemInit() in
// the C startup code
#ifndef __CODERED__
	SystemInit();
#endif

	// initialize USB with HID
	HidDevInfo.idVendor = USB_VENDOR_ID;
	HidDevInfo.idProduct = USB_PROD_ID;
	HidDevInfo.bcdDevice = USB_DEVICE;
	HidDevInfo.StrDescPtr = (uint32_t)&USB_StringDescriptor[0];
	HidDevInfo.InReportCount = 64;	// input packets bytes
	HidDevInfo.OutReportCount = 64;	// output packets bytes
	HidDevInfo.SampleInterval = 10;	// interval timer 10ms for output packets
	HidDevInfo.InReport = GetInReport;
	HidDevInfo.OutReport = SetOutReport;

	DeviceInfo.DevType = USB_DEVICE_CLASS_HUMAN_INTERFACE;
	DeviceInfo.DevDetailPtr = (uint32_t)&HidDevInfo;

	// initialize each block
	InitDamp();
	InitFPGA();
	InitDAC();
	InitVolume();
	InitAdc();
	InitRemocon();
	InitOLED();

	/* Enable Timer32_1, IOCON, and USB blocks (for USB ROM driver) */
	LPC_SYSCON->SYSAHBCLKCTRL |= (EN_TIMER32_1 | EN_IOCON | EN_USBREG);

	/* Use pll and pin init function in rom */
	(*rom)->pUSBD->init_clk_pins();

	/* insert a delay between clk init and usb init */
	for (n = 0; n < 75; n++) {}

	(*rom)->pUSBD->init(&DeviceInfo); /* USB Initialization */
	(*rom)->pUSBD->connect(TRUE);     /* USB Connect */

	while (1)
	{
		__WFI();
		ScanRemocon();
		SCAN_flag = 1;
		ScanDamp();
		SCAN_flag = 0;
		// Timer32.0 scan
		if(TIMER_flag)
		{
			SCAN_flag = 1;
			ScanFPGA();
			TIMER_flag=0;
			SCAN_flag = 0;
		}
		Dispatch();
	}

}


#if defined(__IAR__)
void USBIRQ_IRQHandler()
#else
void USB_IRQHandler()
#endif
{
	(*rom)->pUSBD->isr();
}
