//**********************
// Volume control with SPI
//
// for CS3310, Cirrus Logic
// (C)Copyright 2012 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#include "config.h"
#include "type.h"
#include "gpio.h"
#include "volume.h"
#include "OLED.h"
#include "Damp.h"
#include "remocon.h"
#include "DAC.h"

extern volatile unsigned char Buf[64];
extern volatile unsigned char flag;

volatile unsigned char volume=0;
volatile unsigned char mute=0;

char dB[][4]={"-**","-95","-94","-93","-92","-91","-90",
		"-89","-88","-87","-86","-85","-84","-83","-82","-81","-80",
		"-79","-78","-77","-76","-75","-74","-73","-72","-71","-70",
		"-69","-68","-67","-66","-65","-64","-63","-62","-61","-60",
		"-59","-58","-57","-56","-55","-54","-53","-52","-51","-50",
		"-49","-48","-47","-46","-45","-44","-43","-42","-41","-40",
		"-39","-38","-37","-36","-35","-34","-33","-32","-31","-30",
		"-29","-28","-27","-26","-25","-24","-23","-22","-21","-20",
		"-19","-18","-17","-16","-15","-14","-13","-12","-11","-10",
		"- 9","- 8","- 7","- 6","- 5","- 4","- 3","- 2","- 1","  0",
		      "  1","  2","  3","  4","  5","  6","  7","  8","  9",
		" 10"," 11"," 12"," 13"," 14"," 15"," 16"," 17"," 18"," 19",
		" 20"," 21"," 22"," 23"," 24"," 25"," 26"," 27"," 28"," 29",
		" 30"," 31"
};

void InitVolume()
{

	// PORT0_10
	IOCON_JTAG_TCK_PIO0_10 = IOCON_JTAG_TCK_PIO0_10_FUNC_GPIO | IOCON_JTAG_TCK_PIO0_10_MODE_PULLUP;

	/* Initialize port pins */
	VOLUME_nCS_OUT;		// set output
	VOLUME_SCLK_OUT;	// set output
	VOLUME_SDATAI_OUT;	// set output

	VOLUME_nCS_H;		// nCS active=low pulse
	VOLUME_SCLK_L;		// SCLK
	VOLUME_SDATAI_L;	// SDATAI

	WriteVolume(0x00,0x00);	// software mute

}


void WriteVolume(unsigned char r, unsigned char l)
{
	unsigned char i;

	// when PHONE, volume-10dB
	if(AMP_flag==0){
		if(r>20)r-=20;
		if(l>20)l-=20;
	}

	VOLUME_nCS_L;			// nCS active=low

	for(i=0;i<8;i++)
	{
		if(r&0x80)
		{
			VOLUME_SDATAI_H;
		}
		else
		{
			VOLUME_SDATAI_L;
		}
		VOLUME_SCLK_H;
		r<<=1;
		VOLUME_SCLK_L;
	}

	for(i=0;i<8;i++)
	{
		if(l&0x80)
		{
			VOLUME_SDATAI_H;
		}
		else
		{
			VOLUME_SDATAI_L;
		}
		VOLUME_SCLK_H;
		l<<=1;
		VOLUME_SCLK_L;
	}

	VOLUME_nCS_H;			// nCS active=low

}


void PrintVolume()
{
	unsigned int c;

	if(mute)
	{
		c=RED;
	}
	else
	{
		c=WHITE;
	}
	PrintFont2(10*6,8*7,"V:", c);
	PrintFont2(14*6,8*7,dB[volume/2], c);
	PrintFont2(20*6,8*7,"dB", c);

}

void MuteVolume()
{

	if(VIRTUAL_flag==0)
	{
		return;
	}

	if(mute==0)
	{
		mute=1;
		WriteVolume(0x00,0x00);
		WriteDAC(RESISTER18, 0x51);	// DAC mute on
	}
	else
	{
		mute=0;
		WriteVolume(volume,volume);
		WriteDAC(RESISTER18, 0x50);	// DAC mute off
	}
	// print
	PrintVolume();

	// tell mute via USB
	Buf[2]=mute;
	Buf[1]=0;
	Buf[0]=MUTE_KEY;	// MUTE_KEY
	flag=1;

}


void UpVolume()
{

	if(VIRTUAL_flag==0)
	{
		return;
	}

	// -96dB to -60dB step +5dB
	// -60dB to -40dB step +2dB
	// -40dB to +10dB step +1dB
	if(volume<212)
	{
		if(volume==0){
			volume=2;
		}
		else if(volume<72)
		{
			volume+=10;
		}
		else if(volume<112)
		{
			volume+=4;
		}
		else
		{
			volume+=2;
		}
		if(mute==0)
		{
			WriteVolume(volume,volume);
		}
		// print
		PrintVolume();
		// tell volume via USB
		Buf[2]=volume;
		Buf[1]=0;
		Buf[0]=UP_KEY;	// VOL+
		flag=1;
	}
}


void DownVolume()
{

	if(VIRTUAL_flag==0)
	{
		return;
	}

	// -96dB to -60dB step +5dB
	// -60dB to -40dB step +2dB
	// -40dB to +10dB step +1dB
	if(volume>0)
	{
		if(volume==2){
			volume=0;
		}
		else if(volume<=72)
		{
			volume-=10;
		}
		else if(volume<=112)
		{
			volume-=4;
		}
		else
		{
			volume-=2;
		}
		if(mute==0)
		{
			WriteVolume(volume,volume);
		}
		// print
		PrintVolume();
		// tell volume via USB
		Buf[2]=volume;
		Buf[1]=0;
		Buf[0]=DOWN_KEY;	// VOL-
		flag=1;
	}
}

